package org.freedesktop.dbus;

import java.lang.reflect.Constructor;

/**
 * Represents an error message sent over the Bus.
 * Any errors not associated with a method call are queued by the Bus 
 * and may be retrieved by calling DBusConnection.getError().
 * @see DBusConnection#getError()
 */
class DBusErrorMessage extends DBusMessage
{
   /** The Destination. */
   protected String destination;
   /**
    * Create an error message.
    * @param source The source address.
    * @param destination The destination address.
    * @param name The error name (the type of the error in dot-notation).
    * @param parameters The error parameters (usually a String message).
    * @param serial The serial of the message.
    * @param replyserial The serial of the message this is a reply to.
    */
   DBusErrorMessage(String source, String destination, String name, Object[] parameters, long serial, long replyserial)
   {
      super(source, name, name, parameters, serial, replyserial);
      this.destination = destination;
   }
   /**
    * Create an error message.
    * @param m The message this is a reply to.
    * @param ex Exception that caused this message
    */
   protected DBusErrorMessage(DBusMessage m, DBusExecutionException ex)
   {
      super(null, null, null, new Object[] { ex.getMessage() }, 0, m.getSerial());
      this.type = ex.getClass().getName().replaceAll("[$]", ".");
      this.name = ex.getClass().getName().replaceAll("[$]", ".");
      this.destination = m.getSource();
   }
   /**
    * Create an error message.
    * @param m The message this is a reply to.
    * @param parameters The error parameters (usually a String message).
    */
   protected DBusErrorMessage(DBusMessage m, Object... parameters)
   {
      super(null, null, null, parameters, 0, m.getSerial());
      this.destination = m.getSource();
   }
   /**
    * Create an error message.
    * @param dest The destination to send the error to
    * @param parameters The error parameters (usually a String message).
    */
   protected DBusErrorMessage(String dest, Object... parameters)
   {
      super(null, null, null, parameters, 0, 0);
      this.destination = dest;
   }   /**
    * Create an error message.
    * @param m The message this is a reply to.
    * @param parameters The error parameters (usually a String message).
    * @param ex Exception that caused this message
    */
   protected DBusErrorMessage(DBusMessage m, DBusException ex, Object... parameters)
   {
      super(null, null, null, parameters, 0, m.getSerial());
      this.type = ex.getClass().getName().replaceAll("[$]", ".");
      this.name = ex.getClass().getName().replaceAll("[$]", ".");
      this.destination = m.getSource();
   }
   /**
    * Create an error message.
    * @param destination The destination for the error message.
    * @param parameters The error parameters (usually a String message).
    * @param ex Exception that caused this message
    */
  protected DBusErrorMessage(String destination, DBusException ex, Object... parameters)
   {
      super(null, null, null, parameters, 0);
      this.type = ex.getClass().getName().replaceAll("[$]", ".");
      this.name = ex.getClass().getName().replaceAll("[$]", ".");
      this.destination = destination;
   }
   /**
    * Returns the destination of the error, if any.
    */
   public String getDestination() { return destination; }
   private static native Class createExceptionClass(String name);
   /**
    * Turns this into an exception of the correct type
    */
   public DBusExecutionException getException()
   {
      try {
         Class c = (Class) createExceptionClass(type);
         if (null == c) c = DBusExecutionException.class;
         if (!DBusExecutionException.class.isAssignableFrom(c)) c = DBusExecutionException.class;
         Constructor con = c.getConstructor(String.class);
         if (null == parameters || 0 == parameters.length)
            return (DBusExecutionException) con.newInstance("");
         else {
            String s = "";
            for (Object o: parameters)
               s += o + " ";
            return (DBusExecutionException) con.newInstance(s.trim());
         }
      } catch (Exception e) {
         e.printStackTrace();
         if (null == parameters || 0 == parameters.length)
            return new DBusExecutionException("");
         else {
            String s = "";
            for (Object o: parameters)
               s += o + " ";
            return new DBusExecutionException(s.trim());
         }
      }
   }
   /**
    * Throw this as an exception of the correct type
    */
   public void throwException() throws DBusExecutionException
   {
      throw getException();
   }
}
