package org.freedesktop.dbus;

import java.util.List;
import java.util.Vector;

import java.lang.reflect.Constructor;

/** 
 * A Signal on DBus.
 * Signals should all extend this class. They MUST provide a 0-arg constructor, and any other
 * constructor they provide MUST call the super constructor with signature (String, String, Object...)
 */
public abstract class DBusSignal extends DBusMessage
{
   /** The path to the object this is emitted from */
   protected String objectpath;
   DBusSignal(String source, String objectpath, String type, String name, Object[] parameters, long serial)
   {
      super(source, type, name, parameters, serial);
      this.objectpath = objectpath;
   }
   static DBusSignal createSignal(Class c, String source, String objectpath, long serial, Object[] parameters) throws DBusException
   {
      if (!DBusSignal.class.isAssignableFrom(c)) throw new ClassCastException("Not a DBusSignal");

      Constructor con = c.getDeclaredConstructors()[0];
      Class[] sig = con.getParameterTypes();
      Class[] types = new Class[sig.length-1];
      sig[0] = String.class;
      for (int i = 1; i < sig.length; i++) {
         types[i-1] = sig[i];
      }
      
      try {
         parameters = DBusConnection.deSerialiseParameters(parameters, types);
         Object[] args = new Object[parameters.length + 1];
         int i = 0;
         args[i++] = objectpath;
         for (; i < args.length; i++) {
            args[i] = parameters[i-1];
         }

         return (DBusSignal) con.newInstance(args);
      } catch (Exception e) { 
         e.printStackTrace();
         throw new DBusException(e.getMessage());
      }
   }
   /** 
    * Create a new signal.
    * This contructor MUST be called by all sub classes.
    * @param objectpath The path to the object this is emitted from.
    * @param parameters The parameters of the signal.
    * @throws DBusException This is thrown if the subclass is incorrectly defined.
    */
   protected DBusSignal(String objectpath, Object[] parameters) throws DBusException
   {
      super(null, "", null, parameters, 0);
      try {
         String s = getClass().getName();
         type = s.replaceAll("[$][^$]*$", "").replaceAll("[$]", ".");
         name = s.replaceAll("^.*[$]([^$]*)$", "$1");

         this.objectpath = objectpath;

         // convert recursively everything
         Class[] ts = getClass().getDeclaredConstructors()[0].getParameterTypes();
         Class[] types = new Class[ts.length-1];
         for (int i = 1; i <= types.length; i++) 
            types[i-1] = ts[i];
         if (null != parameters)
            parameters = DBusConnection.convertParameters(parameters, types);

      } catch (Exception e) {
         e.printStackTrace();
         throw new DBusException("Failed to correctly determine DBusSignal type: "+e);
      }
   }
   /** Returns the path to the object this is emitted from. */
   public String getObjectPath() { return objectpath; }

}
